unit statusprogunit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, simpleipc, LResources, Forms, StdCtrls,
  ExtCtrls;

type

  { TStatusForm }

  TStatusForm = class(TForm)
    IPCServer: TSimpleIPCServer;                            //Kommunikation
    Memo1: TMemo;                                           //Zum Bearbeiten, wenn StatusForm aktiv
    FormTimer: TTimer;                                      //Benötigt, um zu schauen, ob Programm noch aktiv, wenn nein, dann Memo=false setzen
    WriteTimer: TTimer;                                     //Ausgabe der gepufferten StatusTexte
    procedure FormClick(Sender: TObject);                   //Memo aktivieren und zum bearbeiten fertig machen
    procedure FormPaint(Sender: TObject);                   //Zeichnen der StatusTexte
    procedure IPCServerMessage(Sender: TObject);            //Steuern des Statusprogramms per Messages
    procedure FormTimerTimer(Sender: TObject);              //Schauen, ob StatusProgr noch aktiv, wenn nein, dann Memo=false setzen
    procedure WriteTimerTimer(Sender: TObject);             //Ausgabe der gepufferten StatusTexte
  private
    FSchreibbar: Boolean;                                   //Kann in die Datei "StatusForm.txt" geschrieben werden
    FStr: array[0..5] of String;                            //Hier werden die Strings, die angezeigt werden sollen zwischengespeichert
    procedure SetSchreibbar(b: Boolean);                    //Öffnet/Schließt die Datei "StatusForm.txt"
    procedure ToFront;                                      //BringToFront funktioniert nicht daher so...
  public
    Sichtbarkeit:byte;
    property Schreibbar: Boolean read FSchreibbar           //Kann in die Datei "StatusForm.txt" geschrieben werden
      write SetSchreibbar;
    constructor Create(AOwner: TComponent); override;       //Constructor
    destructor Destroy; override;                           //Zerstört das StatusFormular
    procedure Write(Str: String);                           //Zeigt Statustext und schreibt in StatusFile
    procedure Exec(Nr: Integer);                            //Führt Befehle aus
  published
  end;

var
  StatusForm: TStatusForm;

implementation

const
  stClose        = 01;                                      //ID zum Schließen des Statusfensters
  stBringToFront = 02;                                      //ID zum in den Vordergrund rücken

var
  StatusFile: TextFile;                                     //Textfile (Status.txt), in dem die writeoperationen ausgeführt werden

procedure TStatusForm.IPCServerMessage(Sender: TObject);    //Steuern des Statusprogramms per Messages
var
  i: Integer;
begin
//  StatusForm.Write('[Status-IPCServerMessage] '+
//    'MsgType['+IntToStr(IPCServer.MsgType)+']: '+
//    IPCServer.StringMessage);

  if IPCServer.MsgType = 1 then
    StatusForm.Write(IPCServer.StringMessage)
  else
    case IPCServer.MsgType of
      2: if TryStrToInt(IPCServer.StringMessage, i) then
           StatusForm.Exec(i);
      else
        StatusForm.Write('[Status-IPCServerMessage] der '+
        'angeforderte Typ ist kein String sondern vom '+
        'Typ['+inttostr(IPCServer.MsgType)+']');
    end;
end;

procedure TStatusForm.FormTimerTimer(Sender: TObject);      //Schauen, ob StatusProgr noch aktiv, wenn nein, dann Memo=false setzen
begin
  if (not StatusForm.Focused) and (not Memo1.Focused) then begin
    FormTimer.Enabled:=false;
    Memo1.Visible:=false;
  end;
end;

procedure TStatusForm.WriteTimerTimer(Sender: TObject);     //Ausgabe der gepufferten StatusTexte
begin
  if (not memo1.Visible) and (Statusform.Visible) then
    Repaint;
end;

procedure TStatusForm.FormClick(Sender: TObject);           //Memo aktivieren und zum Bearbeiten fertig machen
begin
  if not Memo1.Visible then begin
    Schreibbar:=false;
    Memo1.Lines.LoadFromFile('Status.txt');
    Schreibbar:=true;
    FormTimer.Enabled:=true;
    Memo1.Visible:=true;
//    Memo1.Lines.LoadFromFile('Status.txt');
    Memo1.SelStart:=Length(Memo1.Lines.Text)-1;
    Memo1.SetFocus;
  end;
end;

procedure TStatusForm.FormPaint(Sender: TObject);           //Zeichnen der StatusTexte
begin
  if (not memo1.Visible) and (Statusform.Visible) then
    begin
      Canvas.TextOut(2,0,FStr[0]);
      Canvas.TextOut(2,13,FStr[1]);
      Canvas.TextOut(2,26,FStr[2]);
      Canvas.TextOut(2,39,FStr[3]);
      Canvas.TextOut(2,52,FStr[4]);
      Canvas.TextOut(2,65,FStr[5]);
    end;
end;

procedure TStatusForm.SetSchreibbar(b: Boolean);            //Öffnet/Schließt die Datei "StatusForm.txt"
begin
  if b <> FSchreibbar then
    if b then begin
      Append(StatusFile);
      FSchreibbar:=true;
    end else begin
      CloseFile(StatusFile);
      FSchreibbar:=false;
    end;
end;

procedure TStatusForm.ToFront;                              //BringToFront funktioniert nicht daher so...
begin
  BringToFront;
  Repaint;
end;

procedure TStatusForm.Exec(Nr: Integer);                    //Führt Befehle aus
begin
//  StatusForm.Write('[Status-IPCServerMessage] '+
//    'Exec['+IntToStr(Nr)+']');
  case Nr of
    stClose:        Close;
    stBringToFront: ToFront;
  end;
end;

constructor TStatusForm.Create(AOwner: TComponent);         //Constructor

  procedure CreateStatusFile;
  var
    s: String;
  begin
    s:=Application.GetOptionValue('p', 'path');
    if Length(s) = 0 then s:='Status.txt';

    assignfile(StatusFile, s);
    rewrite(StatusFile);
    closefile(StatusFile);
  end;

begin
  inherited create(AOwner);
  Sichtbarkeit:=2;
  FSchreibbar:=False;
  CreateStatusFile;
  Schreibbar:=True;
  IPCServer.Active:=true;
  FStr[0]:='Starte StatusForm';
  FStr[1]:='';
  FStr[2]:='';
  FStr[3]:='';
  FStr[4]:='';
  FStr[5]:='';
  WriteTimer.Enabled:=true;

  Doublebuffered:=True;
end;

destructor TStatusForm.Destroy;                             //Zerstört das StatusFormular
begin
  IPCServer.Active:=false;
  StatusForm.Schreibbar:=true;
  StatusForm.Write('[Status] Das Programm wurde normal geschlossen!');
  StatusForm.Schreibbar:=false;
  inherited;
end;

procedure TStatusForm.Write(Str: String);                   //Zeigt Statustext und schreibt in StatusFile
var
  b:byte;
begin
  if Schreibbar then
    writeln(StatusFile, Str);
  if length(FStr[5]) > 0 then                               //FStr[5] hat einen Inhalt, daher müssen alle Str eins nach vorn verschoben werden
    begin
      FStr[0]:=FStr[1];
      FStr[1]:=FStr[2];
      FStr[2]:=FStr[3];
      FStr[3]:=FStr[4];
      FStr[4]:=FStr[5];
      FStr[5]:=str;
    end
  else                                                      //Zum Programmstart sind noch nicht alle Strings gefüllt, daher erst mal füllen
    begin
      b:=0;
      while length(FStr[b]) > 0 do
        inc(b);
      FStr[b]:=str;
    end;
end;

initialization

{$I statusprogunit.lrs}

end.
