unit statusunit;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, simpleipc, Process, Forms, Dialogs,
  FileUtil;

type

  { TStatus }

  TStatus=class(TComponent)                                 //Statusklasse
    private
      FClient: TSimpleIPCClient;                            //Client zur Kommunikation
      FIsRunning: Boolean;                                  //Läuft Status?!
      FMaxView: Byte;                                       //Sichtbarkeit
      procedure SetMaxView(AValue: Byte);                   //Setzte die Sichtbarkeit
      function ServerActiv: Boolean;                        //Testet 1 Sekunde lang, ob ein Server anwortet
      procedure StatusError(const Msg: String);             //Erstellt einen Error
      procedure SendExec(ID: Integer);                      //Sendet ein Steuerbefehl
    public
      constructor Create(AOwner: TComponent); override; overload;  //Constructor
      constructor Create(AOwner: TComponent; Programm,      //Constructor
        StatusFile: String); overload;
      destructor Destroy; override;                         //Programm schließen
      procedure Write(View: Byte; Str: String); overload;   //Sendet einen Status
      procedure Write(Str: String); overload;               //Sendet einen Status
      procedure BringToFront;                               //Bringt das StatusFenster wieder nach vorn
      property MaxView: Byte read FMaxView write SetMaxView; //Sichtbarkeit
      property IsRunning: Boolean read FIsRunning;          //Läuft der Status ?!
    end;

var
  Status: TStatus;

procedure StatLn(View: Byte; Str: String); overload;        //Sendet einen Status
procedure StatLn(Str: String); overload;                    //Sendet einen Status

implementation

const
  stClose        = 01;                                      //ID zum Schließen des Statusfensters
  stBringToFront = 02;                                      //ID zum in den Vordergrund rücken

var
  AProcess:TProcess;

procedure StatLn(View: Byte; Str: String);                  //Sendet einen Status
begin
  Status.Write(View, Str);
end;

procedure StatLn(Str: String);                              //Sendet einen Status
begin
  Status.Write(Str);
end;

procedure TStatus.Write(View: Byte; Str: String);           //Sendet einen Status
const          //1234567890123456789012345678901234567890
  InStr: String='                                        '; //maximale Tiefe (MaxView) = 20
begin
  if (View <= MaxView) and FClient.ServerRunning then
  begin
    Str:='['+Format('%.2d',[View])+'] '+                    //Anzeige von View
      Copy(InStr, 1, View*2)+                               //für jede Tiefe zwei Leerzeichen einfügen
      Str;                                                  //der anzuzeigende String (Fehlertext / Statustext)

    FClient.Connect;
    FClient.SendStringMessage(Str);
    FClient.Disconnect;
  end;
end;

procedure TStatus.Write(Str: String);                       //Sendet einen Status
begin
  if FClient.ServerRunning then begin
    FClient.Connect;
    FClient.SendStringMessage(Str);
    FClient.Disconnect;
  end;
end;

procedure TStatus.BringToFront;
begin
  SendExec(stBringToFront);                                 //Schließen
end;

procedure TStatus.SetMaxView(AValue: Byte);                 //Setzte die Sichtbarkeit
begin
  if FMaxView = AValue then Exit;
  if AValue > 20 then AValue:=20;
  FMaxView:=AValue;
end;

function TStatus.ServerActiv: Boolean;                      //Testet 1 Sekunde lang, ob ein Server anwortet
var
  b: Byte;
begin
  b:=0;
  while (b < 20) do
    if FClient.ServerRunning then Exit(True)
  else begin
    inc(b);
    Sleep(50);
  end;
  Result:=False;
end;

procedure TStatus.StatusError(const Msg: String);           //Erstellt einen Error
begin
  raise Exception.Create(Msg) at
    get_caller_addr(get_frame),
    get_caller_frame(get_frame);
end;

procedure TStatus.SendExec(ID: Integer);                    //Sendet ein Steuerbefehl
begin
  if FClient.ServerRunning then begin
    FClient.Connect;
    FClient.SendStringMessage(2, IntToStr(ID));
//    FClient.SendStringMessage(2, '0');
    FClient.Disconnect;
  end;
end;

constructor TStatus.Create(AOwner: TComponent);             //Constructor
begin
  Create(AOwner, '', '');
end;

constructor TStatus.Create(AOwner: TComponent; Programm,
  StatusFile: String);

  function StartStatusProg: Boolean;
  begin
    Result:=False;
    try
      AProcess:=TProcess.Create(nil);
      if Length(Programm)=0 then Programm:='StatusProg.exe';
      AProcess.Executable:=UTF8ToConsole(Programm);
      AProcess.Parameters.Add('--path='+UTF8ToConsole(StatusFile));
      AProcess.Execute;
      AProcess.Free;
      Result:=True;
    except
    end;
  end;

begin
  inherited Create(AOwner);
  FIsRunning:=False;
  try
    FClient:=TSimpleIPCClient.Create(Nil);
    FClient.ServerID:='50001';
    MaxView:=2;                                             //Statusanzeigen 0..2 sind sichtbar!!!

    if not FClient.ServerRunning then begin                 //Nur starten, wenn StatusProg (evtl. von einem anderen Programm) noch nicht aktiv

      if not StartStatusProg then StatusError('Error, '+
        'konnte StatusProgramm ['+Programm+'] '+
        'nicht starten, bitte Pfade und Dateinamen überprüfen!');

      if not ServerActiv then StatusError('Server läuft nicht!');

    end;
    FIsRunning:=True;                                       //Es läuft ein Server
  except
    on e: Exception do begin
      ShowMessage('Fehler: '+e.Message);
      Application.Terminate;
    end;
  end;
end;

destructor TStatus.Destroy;                                 //Destructor
begin
  SendExec(stClose);                                        //Schließen
  FClient.Free;
  inherited;
end;

end.

